#ifndef BWIDGETS_FONT_HPP
#define BWIDGETS_FONT_HPP

#include <string>

#include <SDL2/SDL_ttf.h>

#include <basic_widgets/core/type/color.hpp>
#include <basic_widgets/core/type/opaque_struct.hpp>
#include <basic_widgets/core/type/size.hpp>

namespace bwidgets
{
    struct Font final : OpaqueStruct<TTF_Font>
    {
        enum struct Hinting
        {
            LIGHT  = TTF_HINTING_LIGHT,
            MONO   = TTF_HINTING_MONO,
            NONE   = TTF_HINTING_NONE,
            NORMAL = TTF_HINTING_NORMAL
        };

        enum struct RenderMode
        {
            SOLID,
            SHADED,
            BLENDED
        };

        enum struct Style
        {
            BOLD          = TTF_STYLE_BOLD,
            ITALIC        = TTF_STYLE_ITALIC,
            NORMAL        = TTF_STYLE_NORMAL,
            STRIKETHROUGH = TTF_STYLE_STRIKETHROUGH,
            UNDERLINE     = TTF_STYLE_UNDERLINE
        };

        inline static const Color default_color_bg {255, 255, 255, SDL_ALPHA_OPAQUE};
        inline static const Color default_color_fg {0, 0, 0, SDL_ALPHA_OPAQUE};

        const int         ascent;
        const int         descent;
        const long        faces;
        const std::string family_name;
        const bool        fixed_width;
        const int         height;
        const int         line_skip;
        const std::string style_name;

        Font(TTF_Font*);
        Font(const std::string&, int);
        Font(const Font&) = delete;
        ~Font() override;

        auto operator=(const Font&) -> Font& = delete;

        [[nodiscard]] auto hinting() -> Hinting;
        auto               hinting(Hinting) -> Font*;
        [[nodiscard]] auto kerning() -> bool;
        auto               kerning(bool) -> Font*;
        [[nodiscard]] auto outline() -> int;
        auto               outline(int) -> Font*;
        auto render(RenderMode, const std::string&, const Color& fg = default_color_fg,
                    const Color& bg = default_color_bg) -> SDL_Surface*;
        [[nodiscard]] auto style() -> uint8_t;
        auto               style(uint8_t) -> Font*;
        [[nodiscard]] auto text_size(const std::string&) -> Size;

        [[nodiscard]] static auto find(const std::string&) -> std::string;
    };
}

#endif
